"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var webview_utils_common_1 = require("./webview-utils-common");
var WebviewUtilsWKNavigationDelegateImpl = (function (_super) {
    __extends(WebviewUtilsWKNavigationDelegateImpl, _super);
    function WebviewUtilsWKNavigationDelegateImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    WebviewUtilsWKNavigationDelegateImpl.initWithOwnerAndHeaders = function (owner, headers) {
        var handler = WebviewUtilsWKNavigationDelegateImpl.new();
        handler._owner = owner;
        handler.headers = headers;
        return handler;
    };
    WebviewUtilsWKNavigationDelegateImpl.prototype.webViewDecidePolicyForNavigationActionDecisionHandler = function (webView, navigationAction, decisionHandler) {
        var owner = this._owner.get();
        if (!owner || !navigationAction.request.URL) {
            return;
        }
        var isHttpRequest = navigationAction.request.URL.absoluteString.indexOf("http") === 0;
        var areHeadersAdded = true;
        this.headers.forEach(function (val, key) {
            areHeadersAdded = areHeadersAdded && navigationAction.request.valueForHTTPHeaderField(key) === val;
        });
        if (isHttpRequest && !areHeadersAdded) {
            decisionHandler(0);
            var customRequest_1 = new NSMutableURLRequest({
                URL: navigationAction.request.URL,
                cachePolicy: 0,
                timeoutInterval: 60
            });
            for (var i = 0; i < navigationAction.request.allHTTPHeaderFields.count; i++) {
                var headerKey = navigationAction.request.allHTTPHeaderFields.allKeys.objectAtIndex(i);
                customRequest_1.setValueForHTTPHeaderField(navigationAction.request.allHTTPHeaderFields.objectForKey(headerKey), headerKey);
            }
            this.headers.forEach(function (val, key) {
                customRequest_1.setValueForHTTPHeaderField(val, key);
                if (key.toLowerCase() === "user-agent") {
                    webView.customUserAgent = val;
                }
            });
            webView.loadRequest(customRequest_1);
        }
        else {
            decisionHandler(1);
            var navType = "other";
            switch (navigationAction.navigationType) {
                case 0:
                    navType = "linkClicked";
                    break;
                case 1:
                    navType = "formSubmitted";
                    break;
                case 2:
                    navType = "backForward";
                    break;
                case 3:
                    navType = "reload";
                    break;
                case 4:
                    navType = "formResubmitted";
                    break;
            }
            webview_utils_common_1.onLoadStarted(owner, navigationAction.request.URL.absoluteString, navType);
        }
    };
    WebviewUtilsWKNavigationDelegateImpl.prototype.webViewDidFinishNavigation = function (webView, navigation) {
        var owner = this._owner.get();
        if (owner) {
            var src = owner.src;
            if (webView.URL) {
                src = webView.URL.absoluteString;
            }
            webview_utils_common_1.onLoadFinished(owner, src);
        }
    };
    WebviewUtilsWKNavigationDelegateImpl.prototype.webViewDidFailNavigationWithError = function (webView, navigation, error) {
        var owner = this._owner.get();
        if (owner) {
            var src = owner.src;
            if (webView.URL) {
                src = webView.URL.absoluteString;
            }
            webview_utils_common_1.onLoadFinished(owner, src, error.localizedDescription);
        }
    };
    WebviewUtilsWKNavigationDelegateImpl.ObjCProtocols = [WKNavigationDelegate];
    return WebviewUtilsWKNavigationDelegateImpl;
}(NSObject));
var WebViewUtils = (function (_super) {
    __extends(WebViewUtils, _super);
    function WebViewUtils() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    WebViewUtils.addHeaders = function (wv, headers) {
        if (WebViewUtils.isWKWebView(wv)) {
            wv.ios.navigationDelegate = wv._delegate = WebviewUtilsWKNavigationDelegateImpl.initWithOwnerAndHeaders(new WeakRef(wv), headers);
        }
        else {
            wv._delegate = WebViewUtils.initWithOwner(new WeakRef(wv));
            WebViewUtils.headers = headers;
        }
    };
    WebViewUtils.isWKWebView = function (wv) {
        return wv.ios.isKindOfClass(WKWebView.class());
    };
    WebViewUtils.initWithOwner = function (owner) {
        var delegate = new WebViewUtils();
        delegate._owner = owner;
        delegate._originalDelegate = owner.get()._delegate;
        return delegate;
    };
    WebViewUtils.prototype.webViewShouldStartLoadWithRequestNavigationType = function (webView, request, navigationType) {
        var urlString = request.URL.absoluteString;
        var isNavigationTypeBackForward = navigationType === 2;
        var isHttpRequest = urlString.indexOf("http") === 0;
        var areHeadersAdded = true;
        WebViewUtils.headers.forEach(function (val, key) {
            areHeadersAdded = areHeadersAdded && request.valueForHTTPHeaderField(key) === val;
        });
        if (!isHttpRequest || isNavigationTypeBackForward || areHeadersAdded || urlString === "about:blank") {
            return this._originalDelegate.webViewShouldStartLoadWithRequestNavigationType(webView, request, navigationType);
        }
        var nsMutableURLRequest = new NSMutableURLRequest({
            URL: request.URL,
            cachePolicy: 0,
            timeoutInterval: 60
        });
        for (var i = 0; i < request.allHTTPHeaderFields.count; i++) {
            var headerKey = request.allHTTPHeaderFields.allKeys.objectAtIndex(i);
            nsMutableURLRequest.setValueForHTTPHeaderField(request.allHTTPHeaderFields.objectForKey(headerKey), headerKey);
        }
        WebViewUtils.headers.forEach(function (val, key) {
            nsMutableURLRequest.setValueForHTTPHeaderField(val, key);
            if (key.toLowerCase() === "user-agent") {
                NSUserDefaults.standardUserDefaults.registerDefaults(NSDictionary.dictionaryWithObjectForKey(val, "UserAgent"));
            }
        });
        webView.loadRequest(nsMutableURLRequest);
        return false;
    };
    WebViewUtils.prototype.webViewDidStartLoad = function (webView) {
        this._originalDelegate.webViewDidStartLoad(webView);
    };
    WebViewUtils.prototype.webViewDidFinishLoad = function (webView) {
        this._originalDelegate.webViewDidFinishLoad(webView);
    };
    WebViewUtils.prototype.webViewDidFailLoadWithError = function (webView, error) {
        this._originalDelegate.webViewDidFailLoadWithError(webView, error);
    };
    WebViewUtils.ObjCProtocols = [UIWebViewDelegate];
    return WebViewUtils;
}(NSObject));
exports.WebViewUtils = WebViewUtils;
